@@ -3,12 +3,15 @@ package JavaScript::Minifier;
 use strict;
 use warnings;
 
-our $VERSION = '1.14'; # VERSION
+our $VERSION = '1.16'; # VERSION
 
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(minify);
 
+# string 'return' as array to handle special case of returning a regexp from a function
+my @return = qw(r e t u r n);
+
 #return true if the character is allowed in identifier.
 sub isAlphanum {
   return ($_[0] =~ /[\w\$\\]/ || ord($_[0]) > 126);
@@ -139,8 +142,14 @@ sub putLiteral {
       action1($s);
       action1($s);
     }
+    if ($s->{regexp_flag} && $s->{a} eq '[') { # note character class inside regexp, in character class inside regexp there could be unescaped delimiter
+        $s->{charclass_flag} = 1;
+    }
+    elsif ($s->{regexp_flag} && $s->{a} eq ']') {
+        $s->{charclass_flag} = 0;
+    }
     action1($s);
-  } until ($s->{last} eq $delimiter || !defined($s->{a}));
+  } until ($s->{last} eq $delimiter && !($s->{regexp_flag} && $s->{charclass_flag}) || !defined($s->{a}));
   if ($s->{last} ne $delimiter) { # ran off end of file before printing the closing delimiter
     die 'unterminated ' . ($delimiter eq '\'' ? 'single quoted string' : $delimiter eq '"' ? 'double quoted string' : 'regular expression') . ' literal, stopped';
   }
@@ -225,6 +234,10 @@ sub minify {
   $s->{b} = _get($s);
   $s->{c} = _get($s);
   $s->{d} = _get($s);
+  $s->{regexp_flag} = 0;
+  $s->{charclass_flag} = 0;
+  $s->{return_flag} = 0;
+  $s->{return_string} = '';
   $s->{last} = undef; # assign for safety
   $s->{lastnws} = undef; # assign for safety
 
@@ -237,6 +250,12 @@ sub minify {
       die 'minifier bug: minify while loop starting with whitespace, stopped';
     }
 
+    # track 'return' operator
+    if ($s->{a} ne '/') {
+        $s->{return_flag} = defined($return[length($s->{return_string})]) && $s->{a} eq $return[length($s->{return_string})];
+        $s->{return_string} = $s->{return_flag} ? $s->{return_string} . $s->{a} : '';
+    }
+
     # Each branch handles trailing whitespace and ensures $s->{a} is on non-whitespace or undef when branch finishes
     if ($s->{a} eq '/') { # a division, comment, or regexp literal
       if (defined($s->{b}) && $s->{b} eq '/') { # slash-slash comment
@@ -292,14 +311,16 @@ sub minify {
           die 'unterminated comment, stopped';
         }
       }
-      elsif (defined($s->{lastnws}) && ($s->{lastnws} eq ')' || $s->{lastnws} eq ']' ||
-                                        $s->{lastnws} eq '.' || isAlphanum($s->{lastnws}))) { # division
+      elsif ((defined($s->{lastnws}) && ($s->{lastnws} eq ')' || $s->{lastnws} eq ']' ||
+                                        $s->{lastnws} eq '.' || isAlphanum($s->{lastnws}))) && (!$s->{return_flag} || length($s->{return_string}) != scalar(@return))) { # division
+
         action1($s);
         collapseWhitespace($s);
         # don't want a division to become a slash-slash comment with following conditional comment
         onWhitespaceConditionalComment($s) ? action1($s) : preserveEndspace($s);
       }
       else { # regexp literal
+        $s->{regexp_flag} = 1;
         putLiteral($s);
         collapseWhitespace($s);
         # don't want closing delimiter to become a slash-slash comment with following conditional comment
@@ -307,6 +328,7 @@ sub minify {
       }
     }
     elsif ($s->{a} eq '\'' || $s->{a} eq '"' ) { # string literal
+      $s->{regexp_flag} = 0;
       putLiteral($s);
       preserveEndspace($s);
     }
@@ -433,6 +455,8 @@ Miller 'tmhall' Hall
 
 Вячеслав 'vti' Тихановский
 
+Fedor A. 'faf' Fetisov
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2007 by Peter Michaux
